package com.gic.udesk.service.outer.impl;

import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.udesk.constant.UdeskStatusEnum;
import com.gic.udesk.dto.UdeskTicketDetailDTO;
import com.gic.udesk.dto.customfield.UdeskCustomFieldOptionDTO;
import com.gic.udesk.dto.gic.*;
import com.gic.udesk.entity.TabUdeskCustomers;
import com.gic.udesk.entity.TabUdeskEnterprise;
import com.gic.udesk.qo.TicketListQO;
import com.gic.udesk.service.UdeskCustomersService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.udesk.service.GicTicketApiService;
import com.gic.udesk.service.UdeskBaseApiService;
import com.gic.udesk.service.UdeskEnterpriseService;
import com.gic.udesk.utils.UdeskConstant;
import com.gic.udesk.utils.UdeskHttpClientUtil;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service("gicTicketApiService")
public class GicTicketApiServiceImpl implements GicTicketApiService{
    private static final Logger LOGGER = LogManager.getLogger(GicTicketApiServiceImpl.class);

    @Autowired
    private UdeskBaseApiService udeskBaseApiService;
    @Autowired
    private UdeskEnterpriseService udeskEnterpriseService;
    @Autowired
    private UdeskCustomersService udeskCustomersService;

    /**
     * udesk工单接口 (创建)
     */
    private static final String UDESK_CREATE_TICKET = "/tickets/";

    /**
     * udesk工单接口 (更新)
     */
    private static final String UDESK_UPDATE_TICKET = "/tickets/";
    /**
     * udesk工单详情接口路径
     */
    private static final String TICKET_DETAIL = "/tickets/detail";

    /**
     * 客户
     */
    private static final String CUSTOMERS = "/customers";

    @Override
    public ServiceResponse<Integer> saveTicket(GicTicketDTO gicTicketDTO) {
        Integer businessModuleType = gicTicketDTO.getBusinessModuleType();
        boolean isNotRightBusinessModuleType = businessModuleType == null || (businessModuleType != 1 && businessModuleType != 2);
        if (isNotRightBusinessModuleType) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "模块类型错误，数值非法");
        }
        //设置模板ID
        if (businessModuleType.intValue() == 1) {
            gicTicketDTO.setTemplate_id(UdeskConstant.GIC_BASE_MODULE_TEMPLATE_ID);
        } else {
            gicTicketDTO.setTemplate_id(UdeskConstant.APP_MODULE_TEMPLATE_ID);
        }

        //查询有无客户ID
        Integer gicEnterpriseId = gicTicketDTO.getGicEnterpriseId();
        Integer gicUserId = gicTicketDTO.getGicUserId();
        String gicUserName = gicTicketDTO.getGicUserName();

        TabUdeskCustomers obj = udeskCustomersService.hasCustomerId(gicEnterpriseId, gicUserId);
        if (obj != null) {
            gicTicketDTO.setType("customer_id");
            gicTicketDTO.setType_content(obj.getCustomerId().toString());
        } else {
            //创建客户
            JSONObject customer = new JSONObject();
            JSONObject customerSub = new JSONObject();
            //客户名称
            customerSub.put("nick_name", gicUserName);
            customer.put("customer", customerSub);
            LOGGER.info("gic新建udesk客户,内容为:{}", customer.toJSONString());
            String customerStr = UdeskHttpClientUtil
                    .postUdeskJsonFromJson(CUSTOMERS, customer.toJSONString());
            LOGGER.info("gic新建udesk客户,返回为:{}", customerStr);
            JSONObject customerRes = JSONObject.parseObject(customerStr);
            if (customerRes != null) {
                int code = customerRes.getIntValue("code");
                JSONObject customerObj = customerRes.getJSONObject("customer");
                if (code == UdeskConstant.UDESK_SUCCESS_CODE) {
                    int customerId = customerObj.getIntValue("id");

                    gicTicketDTO.setType("customer_id");
                    gicTicketDTO.setType_content(customerId + "");

                    //保存到
                    udeskCustomersService.save(gicEnterpriseId, customerId, gicUserId);
                } else {
                    return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), customerRes.getString("message"));
                }
            }
        }

        Map<String, Object> paramMap = new HashMap<>(4);
        paramMap.put("ticket", gicTicketDTO);
        String params = JSON.toJSONString(paramMap);
        LOGGER.info("gic新建udesk工单,内容为:{}", params);
        String respStr = UdeskHttpClientUtil
                .postUdeskJsonFromJson(UDESK_CREATE_TICKET, params);
        LOGGER.info("gic新建udesk工单,工单返回为:{}", respStr);
        TicketCreateResDTO createResDTO = JSON.parseObject(respStr, TicketCreateResDTO.class);
        if (createResDTO != null && createResDTO.getCode() == UdeskConstant.UDESK_SUCCESS_CODE) {
            Integer ticketId = createResDTO.getTicket_id();
            //udesk工单详情接口
            String detailStr = this.getUdeskTicketDetail(ticketId.toString());
            UdeskTicketDetailDTO detail = JSON.parseObject(detailStr, UdeskTicketDetailDTO.class);
            if (detail.getCode() == UdeskConstant.UDESK_SUCCESS_CODE) {
                TabUdeskEnterprise record = new TabUdeskEnterprise();
                //工单ID
                record.setTicketId(ticketId);
                record.setEnterpriseId(gicEnterpriseId);
                //工单编号
                record.setTicketNum(detail.getTicket().getField_num());
                //工单标题
                record.setTicketTitle(gicTicketDTO.getSubject());
                //业务模块类型 1:基础模块 2:应用模块
                record.setBusinessType(gicTicketDTO.getBusinessModuleType());
                //业务模块ID
                if (businessModuleType.intValue() == 1) {
                    String businessModuleValue = gicTicketDTO.getTicket_field().get(UdeskConstant.GIC_BASE_MODULE_SELECT);
                    record.setBusinessModule(businessModuleValue);
                    record.setBusinessModuleName(getBusinessModuleName(businessModuleValue));
                } else {
                    record.setBusinessModule(gicTicketDTO.getTicket_field().get(UdeskConstant.APP_MODULE_NAME));
                    record.setBusinessModuleName(gicTicketDTO.getTicket_field().get(UdeskConstant.APP_MODULE_NAME));
                }
                udeskEnterpriseService.save(record);
                return ServiceResponse.success(ticketId);
            } else {
                return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), detail.getMessage());
            }
        } else {
            return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), createResDTO.getMessage());
        }
    }

    @Override
    public ServiceResponse<Page<UdeskEnterpriseDTO>> page(TicketListQO params) {
        com.github.pagehelper.Page<TabUdeskEnterprise> page = udeskEnterpriseService.page(params);
        Page<UdeskEnterpriseDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, UdeskEnterpriseDTO.class);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<TicketRepliesDTO> getTicketReplies(Integer ticketId, Integer currentPage, Integer pageSize) {
        LOGGER.info(" 查询工单回复信息,工单Id:{}", ticketId);
        Map<String, Object> params = new HashMap<>(4);
        params.put("page", currentPage);
        params.put("per_page", pageSize);
        String respStr = UdeskHttpClientUtil
                .getUdeskJsonFromParam("/tickets/" + ticketId + "/replies", params);
        LOGGER.info("查询工单回复信息,工单返回为:{}", respStr);
        TicketRepliesDTO updateDetail = JSON.parseObject(respStr, TicketRepliesDTO.class);
        if (updateDetail.getCode() == UdeskConstant.UDESK_SUCCESS_CODE) {
            List<TicketRepliesDataDTO> replies = updateDetail.getReplies();
            List<TicketRepliesDataDTO> repliesTemp = new ArrayList<>();
            for (TicketRepliesDataDTO dataDTO : replies) {
                //外部评论才展示
                String needIgnoreStr = "TAPD的bug链接为";
                //tapd的链接屏蔽
                if (!dataDTO.getContent().contains(needIgnoreStr)) {
                    repliesTemp.add(dataDTO);
                }
            }
            updateDetail.setReplies(repliesTemp);
            return ServiceResponse.success(updateDetail);
        }
        return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "无数据");
    }

    @Override
    public ServiceResponse<String> closeTicket(Integer ticketId) {
        JSONObject param = new JSONObject();
        JSONObject updateJson = new JSONObject();
        updateJson.put("status_id", UdeskStatusEnum.CLOSED.getValue());
        param.put("ticket", updateJson);
        LOGGER.info("更新udesk工单,工单Id:{},更新内容为:{}", ticketId, param.toJSONString());
        String updateTicketRespStr = UdeskHttpClientUtil
                .putUdeskJsonFromJson(UDESK_UPDATE_TICKET + ticketId, param.toJSONString());
        LOGGER.info("更新udesk工单,工单返回为:{}", updateTicketRespStr);
        UdeskTicketDetailDTO updateDetail = JSON.parseObject(updateTicketRespStr, UdeskTicketDetailDTO.class);
        if (updateDetail != null && updateDetail.getCode() == UdeskConstant.UDESK_SUCCESS_CODE) {
            //同步gic工单状态
            udeskEnterpriseService.close(ticketId);

            //todo 同步到tapd关闭

            return ServiceResponse.success(updateDetail.getMessage());
        }
        return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), updateDetail.getMessage());
    }

    @Override
    public ServiceResponse<UdeskTicketDetailDTO> getTicketDetailById(Integer ticketId) {
        //udesk工单详情接口
        String detailStr = this.getUdeskTicketDetail(ticketId.toString());
        UdeskTicketDetailDTO detail = JSON.parseObject(detailStr, UdeskTicketDetailDTO.class);
        if (detail.getCode() == UdeskConstant.UDESK_SUCCESS_CODE) {
            return ServiceResponse.success(detail);
        }
        return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), detail.getMessage());
    }

    /**
     * udesk工单详情接口
     * @param id
     * @return
     */
    private String getUdeskTicketDetail(String id) {
        Map<String, Object> param = new HashMap<>(8);
        param.put("id", id);
        LOGGER.info("请求udesk工单详情入参id为{}", id);
        String detailStr = UdeskHttpClientUtil.getUdeskJsonFromParam(TICKET_DETAIL, param);
        LOGGER.info("请求udesk工单详情返回为{}", detailStr);
        return detailStr;
    }

    /**
     * gic基础模块 级联名称获取
     * @param businessModule
     * @return
     */
    private String getBusinessModuleName(String businessModule) {
        String[] arr = businessModule.split(",");
        //第一层级
        String name1 = "";
        //第二层级
        String name2 = "";
        ServiceResponse<List<UdeskCustomFieldOptionDTO>> result = udeskBaseApiService.getGicServiceSelect();
        if (result.isSuccess()) {
            List<UdeskCustomFieldOptionDTO> list = result.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                for (UdeskCustomFieldOptionDTO optionDTO : list) {
                    if (optionDTO.getValue().equals(arr[0])) {
                        name1 = optionDTO.getTitle();
                        if (CollectionUtils.isNotEmpty(optionDTO.getSubs())) {
                            for (UdeskCustomFieldOptionDTO subDTO : optionDTO.getSubs()) {
                                if (subDTO.getValue().equals(arr[1])) {
                                    name2 = subDTO.getTitle();
                                    break;
                                }
                            }
                        }
                        break;
                    }
                }
                return name1 + "/" + name2;
            }
        }
        return null;
    }

}
