package com.gic.udesk.service.outer.impl;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.validation.ConstraintViolationException;

import com.gic.udesk.constant.UdeskStatusEnum;
import com.gic.udesk.entity.TabUdeskTapdRel;
import com.gic.udesk.service.*;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.udesk.dto.*;
import com.gic.udesk.entity.TabUdeskMessage;
import com.gic.udesk.entity.TabUdeskTicket;
import com.gic.udesk.utils.UdeskConstant;
import com.gic.udesk.utils.UdeskHttpClientUtil;

@Service("udeskMessageApiService")
public class UdeskMessageApiServiceImpl implements UdeskMessageApiService {
    private static Logger logger = LogManager.getLogger(UdeskMessageApiServiceImpl.class);

    @Autowired
    private UdeskMessageService udeskMessageService;
    @Autowired
    private UdeskBaseApiService udeskBaseApiService;
    @Autowired
    private UdeskTicketService udeskTicketService;
    @Autowired
    private UdeskTapdRelService udeskTapdRelService;
    @Autowired
    private UdeskEnterpriseService udeskEnterpriseService;

    @Resource(name = "udeskTapdRelMap")
    private Map<String, String> udeskTapdRelMap;
    /**
     * udesk工单详情接口路径
     */
    private static final String TICKET_DETAIL = "/tickets/detail";

    /**
     * udesk客户公司详情接口路径
     */
    private static final String ORG_DETAIL = "/organizations/show_org";

    /**
     * tapd 缺陷
     */
    private static final String TAPD_BUGS = "/bugs";

    private static final String TAPD_BUGS_URL_PART = "/bugtrace/bugs/view?bug_id=";

    /**
     * udesk客服回复接口路径
     */
    private static final String AGENT_REPLY = "/tickets/agent_reply";

    @Override
    public ServiceResponse<String> saveUdeskMessage(UdeskMessageDTO messageDTO) {
        TabUdeskMessage tabUdeskMessage = new TabUdeskMessage();
        tabUdeskMessage.setEvent(messageDTO.getEvent());
        tabUdeskMessage.setMsgBody(messageDTO.getMsgBody());
        tabUdeskMessage.setSource(messageDTO.getSource());
        tabUdeskMessage.setStatus(UdeskConstant.UNDELETED);
        tabUdeskMessage.setTopic(messageDTO.getTopic());
        int messageId = udeskMessageService.save(tabUdeskMessage);
        //设置主键，用于update
        tabUdeskMessage.setId(messageId);
        logger.info("消息为{}已经保存成功", messageDTO);
        //创建工单消息
        if (UdeskConstant.TICKET_CREATE.equals(messageDTO.getEvent())) {
            //udesk工单ID
            String id = messageDTO.getMsgBody();
            //udesk工单详情接口
            String detailStr = this.getUdeskTicketDetail(id);
            UdeskTicketDetailDTO detail = JSON.parseObject(detailStr, UdeskTicketDetailDTO.class);
            if (detail.getCode() == UdeskConstant.UDESK_SUCCESS_CODE) {
                //usesk工单信息
                TicketDTO ticket = detail.getTicket();
                //udesk自定义字段信息
                Map<String, String> customFields = ticket.getCustom_fields();
                //udesk工单表实体
                TabUdeskTicket tabUdeskTicket = new TabUdeskTicket();
                TapdBugDTO tapdBug = new TapdBugDTO();
                if (customFields != null) {
                    //udesk客服名称--tapd创建人
                    String staffValue = customFields.get(UdeskConstant.UDESK_FIELD_STAFF);
                    tabUdeskTicket
                            .setServiceStaff(udeskTapdRelMap.get(UdeskConstant.UDESK_FIELD_STAFF + ":" + staffValue));
                    //tapd项目ID
                    String workspaceValue = customFields.get(UdeskConstant.UDESK_FIELD_WORKSPACE);
                    tabUdeskTicket.setWorkspaceId(
                            udeskTapdRelMap.get(UdeskConstant.UDESK_FIELD_WORKSPACE + ":" + workspaceValue));
                    //TPAD当前处理人
                    String transactorValue = customFields.get(UdeskConstant.UDESK_FIELD_TRANSACTOR);
                    if (StringUtils.isNotBlank(transactorValue)) {
                        String[] transactors = transactorValue.split(",");
                        StringBuilder builder = new StringBuilder();
                        for (String transactor : transactors) {
                            builder.append(udeskTapdRelMap.get(UdeskConstant.UDESK_FIELD_TRANSACTOR + ":" + transactor))
                                    .append(";");
                        }
                        tabUdeskTicket.setCurrentOwner(builder.toString());
                    }
                    //填充部分自定义字段值
                    this.setUdeskCostomFields(tabUdeskTicket, tapdBug, customFields);
                }
                //受理客服id
                tabUdeskTicket.setAssigneeId(ticket.getAssignee_id());
                //受理客服姓名
                tabUdeskTicket.setAssigneeName(ticket.getAssignee_name());
                //创建人id
                tabUdeskTicket.setCreatorId(ticket.getCreator_id());
                //工单编号
                tabUdeskTicket.setFieldNum(ticket.getField_num());
                //工单id
                tabUdeskTicket.setId(ticket.getId());
                //客户所属公司id
                Integer organizationId = ticket.getOrganization_id();
                if (organizationId != null) {
                    this.setOrganizationIdAndName(tabUdeskTicket, organizationId);
                }
                //填充工单优先级中文名称
                this.setPriority(ticket, tabUdeskTicket, tapdBug);
                tabUdeskTicket.setStatus(UdeskConstant.UNDELETED);
                //状态英文名称
                tabUdeskTicket.setStatusEn(ticket.getStatus_en());
                //tapd状态值
                tabUdeskTicket.setStatusTapd(udeskTapdRelMap.get(ticket.getStatus_en()));
                //标题
                tabUdeskTicket.setSubject(ticket.getSubject());
                //客户id
                tabUdeskTicket.setUserId(ticket.getUser_id());
                //客户姓名
                tabUdeskTicket.setUserName(ticket.getUser_name());
                try {
                    udeskTicketService.save(tabUdeskTicket);

                    updateGicTicketStatus(Integer.parseInt(id), ticket.getStatus_en());
                    logger.info("工单id为{}已经保存成功", ticket.getId());
                } catch (DataIntegrityViolationException | ConstraintViolationException e) {
                    logger.info("工单id为{}已经保存失败，重复插入", ticket.getId(), e);
                    return null;
                }
                tapdBug.setTitle(ticket.getSubject());
                tapdBug.setCurrent_owner(tabUdeskTicket.getCurrentOwner());
                tapdBug.setReporter(UdeskConstant.TPAD_REPORTER);

                String workspaceId = tabUdeskTicket.getWorkspaceId();
                tapdBug.setWorkspace_id(workspaceId);
                this.setTapdCustomField(workspaceId, tapdBug, tabUdeskTicket);
                tapdBug.setStatus(tabUdeskTicket.getStatusTapd());

                this.setDescription(ticket, tapdBug);

                String bugStr = JSON.toJSONString(tapdBug);
                logger.info("添加TAPDbug入参为{}", bugStr);
                String bugReturnStr = UdeskHttpClientUtil.postTapdJsonFromJson(TAPD_BUGS, bugStr);
                logger.info("添加TAPDbug返回为{}", bugReturnStr);
                TapdRespBugDTO tapdRespBug = JSON.parseObject(bugReturnStr, TapdRespBugDTO.class);
                if (tapdRespBug.getStatus() == UdeskConstant.TAPD_RESP_SUCCESS_STATUS) {
                    //tapd缺陷ID
                    String bugId = tapdRespBug.getData().getBug().getId();
                    tabUdeskTicket.setBugId(bugId);
                    udeskTicketService.update(tabUdeskTicket);
                    String bugUrl = UdeskConstant.TAPD_HOST + workspaceId + TAPD_BUGS_URL_PART + bugId;
                    //客服回复对象
                    UdeskAgentReplyDTO reply = new UdeskAgentReplyDTO();
                    //客服id
                    reply.setAgent_id(Integer.valueOf(udeskTapdRelMap.get(UdeskConstant.UDESK_DEFAULT_AGENT)));
                    //回复类型,"internal"内部备注／"external"外部评论
                    reply.setFunction_type(UdeskConstant.INTERNAL);
                    //工单ID
                    reply.setId(ticket.getId());
                    //回复内容
                    reply.setNew_reply("TAPD的bug链接为：<a href=\""+ bugUrl +"\" target=\"_blank\">" + bugUrl + "</a>");
                    //客服回复对象转为接口入参数
                    String replyStr = JSON.toJSONString(reply);
                    logger.info("回复udesk工单bug链接入参为{}", replyStr);
                    String replyRespStr = UdeskHttpClientUtil.postUdeskJsonFromJson(AGENT_REPLY, replyStr);
                    logger.info("回复udesk工单bug链接返回为{}", replyRespStr);
                    UdeskRespReplyDTO replyDetail = JSON.parseObject(replyRespStr, UdeskRespReplyDTO.class);
                    if (replyDetail.getCode() != UdeskConstant.UDESK_SUCCESS_CODE) {
                        String exceptionRecord = "工单id为" + id + "回复udesk工单bug链接失败，返回为" + replyRespStr;
                        logger.info(exceptionRecord);
                        udeskBaseApiService.saveUdeskException(exceptionRecord);
                    } else {
                        //消息删除status = 0
                        this.updateTabUdeskMessage(tabUdeskMessage, id, bugId);
                    }
                    if(!"new".equals(tapdBug.getStatus())){
                        this.reSetTapdStatus(id, ticket, tapdBug, workspaceId, bugId);
                    }
                    return ServiceResponse.success(bugId);
                } else {
                    String exceptionRecord = "工单id为" + id + "创建TAPDbug失败，返回为" + bugReturnStr;
                    logger.info(exceptionRecord);
                    udeskBaseApiService.saveTapdException(exceptionRecord);
                }
            } else {
                String exceptionRecord = "工单id为" + id + "请求udesk工单详情异常，返回为" + detailStr;
                logger.info(exceptionRecord);
                udeskBaseApiService.saveUdeskException(exceptionRecord);
            }
        } else if (UdeskConstant.TICKET_UPDATE.equals(messageDTO.getEvent())) {
            //更新消息
            String id = messageDTO.getMsgBody();
            String detailStr = this.getUdeskTicketDetail(id);
            UdeskTicketDetailDTO detail = JSON.parseObject(detailStr, UdeskTicketDetailDTO.class);
            logger.info("工单详情{}", detail != null ? JSON.toJSONString(detail) : null);
            if (detail.getCode() == UdeskConstant.UDESK_SUCCESS_CODE) {
                //udesk接口查询的工单信息
                TicketDTO ticket = detail.getTicket();
                //数据库查询的工单记录信息
                TabUdeskTicket tabUdeskTicket = udeskTicketService.getById(Integer.parseInt(id));
                if (tabUdeskTicket == null) {
                    UdeskMessageDTO createMessage = new UdeskMessageDTO();
                    //事件类型 创建/更新
                    createMessage.setEvent(UdeskConstant.TICKET_CREATE);
                    //工单ID
                    createMessage.setMsgBody(id);
                    createMessage.setSource(UdeskConstant.UDESK_UPDATE_CREATE);
                    //消息主题
                    createMessage.setTopic(UdeskConstant.UDESK_TICKET_TOPIC);
                    ServiceResponse<String> saveResult = this.saveUdeskMessage(createMessage);
                    String bugId = saveResult.getResult();
                    //删除消息status = 0
                    this.updateTabUdeskMessage(tabUdeskMessage, id, bugId);
                    return ServiceResponse.success(bugId);
                }
                TapdBugUpdateDTO tapdBug = new TapdBugUpdateDTO();
                //防止创建工单误触发更新
                String bugId = tabUdeskTicket.getBugId();
                if (StringUtils.isBlank(bugId)) {
                    return null;
                }
                tapdBug.setId(bugId);
                tapdBug.setTitle(ticket.getSubject());
                tapdBug.setCurrent_user(UdeskConstant.TPAD_REPORTER);

                this.setDescription(ticket, tapdBug);
                //tapd项目ID
                String workspaceId = tabUdeskTicket.getWorkspaceId();
                tapdBug.setProject_id(Integer.valueOf(workspaceId));
                tapdBug.setWorkspace_id(workspaceId);
                tapdBug.setReporter(UdeskConstant.TPAD_REPORTER);
                //工单状态英文名称
                String statusEn = ticket.getStatus_en();

                //同步到gic工单状态
                logger.info("同步gic工单状态，ID：{}, 状态：{}", Integer.parseInt(id), statusEn);
                updateGicTicketStatus(Integer.parseInt(id), statusEn);
                //如果udesk工单状态不等于本地数据库对应的工单状态
                if (StringUtils.isNotBlank(statusEn) && !statusEn.equals(tabUdeskTicket.getStatusEn())) {
                    if (UdeskConstant.CLOSED.equals(tabUdeskTicket.getStatusEn())) {
                        //本地工单状态如果是关闭状态，设置tapd缺陷 更新人（操作人）
                        tapdBug.setCurrent_owner(tabUdeskTicket.getCurrentOwner());
                    }
                    //获取字典的状态值
                    String tapdStatus = udeskTapdRelMap.get(statusEn);
                    tabUdeskTicket.setStatusEn(statusEn);
                    tabUdeskTicket.setStatusTapd(tapdStatus);
                    //同步tapd缺陷状态
                    tapdBug.setStatus(tapdStatus);
                }
                tabUdeskTicket.setSubject(ticket.getSubject());

                this.setPriority(ticket, tabUdeskTicket, tapdBug);
                Integer organizationId = ticket.getOrganization_id();
                if(organizationId != null && !organizationId.equals(tabUdeskTicket.getOrganizationId())){
                    this.setOrganizationIdAndName(tabUdeskTicket, organizationId);
                }

                Map<String, String> customFields = ticket.getCustom_fields();
                if(customFields != null){
                    this.setUdeskCostomFields(tabUdeskTicket, tapdBug, customFields);
                }
                tabUdeskTicket.setUserName(ticket.getUser_name());
                this.setTapdCustomField(workspaceId, tapdBug, tabUdeskTicket);

                udeskTicketService.update(tabUdeskTicket);
                String bugStr = JSON.toJSONString(tapdBug);
                logger.info("更新TAPDbug入参为{}", bugStr);
                String bugReturnStr = UdeskHttpClientUtil.postTapdJsonFromJson(TAPD_BUGS, bugStr);
                logger.info("更新TAPDbug返回为{}", bugReturnStr);
                TapdRespBugDTO tapdRespBug = JSON.parseObject(bugReturnStr, TapdRespBugDTO.class);
                if (tapdRespBug.getStatus() == UdeskConstant.TAPD_RESP_SUCCESS_STATUS) {
                    this.updateTabUdeskMessage(tabUdeskMessage, id, bugId);
                    return ServiceResponse.success(bugId);
                } else {
                    String exceptionRecord = "工单id为" + id + "更新TAPDbug失败，返回为" + bugReturnStr;
                    logger.info(exceptionRecord);
                    udeskBaseApiService.saveTapdException(exceptionRecord);
                }
            } else {
                String exceptionRecord = "工单id为" + id + "更新工单请求udesk工单详情异常，返回为" + detailStr;
                logger.info(exceptionRecord);
                udeskBaseApiService.saveUdeskException(exceptionRecord);
            }
        }
        return null;
    }

    @Override
    public ServiceResponse<String> saveUdeskMessageNoTapd(UdeskMessageDTO messageDTO) {
        //创建工单消息
        if (UdeskConstant.TICKET_CREATE.equals(messageDTO.getEvent())) {
            //udesk工单ID
            String id = messageDTO.getMsgBody();
            //udesk工单详情接口
            String detailStr = this.getUdeskTicketDetail(id);
            UdeskTicketDetailDTO detail = JSON.parseObject(detailStr, UdeskTicketDetailDTO.class);
            if (detail.getCode() == UdeskConstant.UDESK_SUCCESS_CODE) {
                //usesk工单信息
                TicketDTO ticket = detail.getTicket();
                //只修改gic状态，不生成本地工单数据，因为和同步到tapd触发器会冲突
                updateGicTicketStatus(Integer.parseInt(id), ticket.getStatus_en());
            } else {
                String exceptionRecord = "工单id为" + id + "请求udesk工单详情异常，返回为" + detailStr;
                logger.info(exceptionRecord);
                udeskBaseApiService.saveUdeskException(exceptionRecord);
            }
        } else if (UdeskConstant.TICKET_UPDATE.equals(messageDTO.getEvent())) {
            //更新消息
            String id = messageDTO.getMsgBody();
            String detailStr = this.getUdeskTicketDetail(id);
            UdeskTicketDetailDTO detail = JSON.parseObject(detailStr, UdeskTicketDetailDTO.class);
            logger.info("工单详情{}", detail != null ? JSON.toJSONString(detail) : null);
            if (detail.getCode() == UdeskConstant.UDESK_SUCCESS_CODE) {
                //udesk接口查询的工单信息
                TicketDTO ticket = detail.getTicket();
                //工单状态英文名称
                String statusEn = ticket.getStatus_en();

                //同步到gic工单状态
                updateGicTicketStatus(Integer.parseInt(id), statusEn);
            } else {
                String exceptionRecord = "工单id为" + id + "更新工单请求udesk工单详情异常，返回为" + detailStr;
                logger.info(exceptionRecord);
                udeskBaseApiService.saveUdeskException(exceptionRecord);
            }
        }
        return ServiceResponse.success();
    }

    /**
     * udesk工单详情接口
     * @param id
     * @return
     */
    private String getUdeskTicketDetail(String id) {
        Map<String, Object> param = new HashMap<>(8);
        param.put("id", id);
        logger.info("请求udesk工单详情入参id为{}", id);
        String detailStr = UdeskHttpClientUtil.getUdeskJsonFromParam(TICKET_DETAIL, param);
        logger.info("请求udesk工单详情返回为{}", detailStr);
        return detailStr;
    }

    /**
     * 设置部分自定义字段的值到udesk工单
     * @param tabUdeskTicket
     * @param tapdBug
     * @param customFields
     */
    private void setUdeskCostomFields(TabUdeskTicket tabUdeskTicket, TapdBugBaseDTO tapdBug,
            Map<String, String> customFields) {
        //严重程度
        String severityValue = customFields.get(UdeskConstant.UDESK_FIELD_SEVERITY);
        tabUdeskTicket.setSeverity(udeskTapdRelMap.get(UdeskConstant.UDESK_FIELD_SEVERITY + ":" + severityValue));
        tapdBug.setSeverity(tabUdeskTicket.getSeverity());
        //发现阶段
        String originphaseValue = customFields.get(UdeskConstant.UDESK_FIELD_ORIGINPHASE);
        tapdBug.setOriginphase(udeskTapdRelMap.get(UdeskConstant.UDESK_FIELD_ORIGINPHASE + ":" + originphaseValue));
        //测试类型
        String testtypeValue = customFields.get(UdeskConstant.UDESK_FIELD_TESTTYPE);
        tapdBug.setTesttype(udeskTapdRelMap.get(UdeskConstant.UDESK_FIELD_TESTTYPE + ":" + testtypeValue));
        //缺陷类型
        String bugtypeValue = customFields.get(UdeskConstant.UDESK_FIELD_BUGTYPE);
        tapdBug.setBugtype(udeskTapdRelMap.get(UdeskConstant.UDESK_FIELD_BUGTYPE + ":" + bugtypeValue));
        //重现规律
        String frequencyValue = customFields.get(UdeskConstant.UDESK_FIELD_FREQUENCY);
        tapdBug.setFrequency(udeskTapdRelMap.get(UdeskConstant.UDESK_FIELD_FREQUENCY + ":" + frequencyValue));
    }

    /**
     * 工单表填充客户所属公司名称
     * @param tabUdeskTicket
     * @param organizationId 客户所属公司ID
     */
    private void setOrganizationIdAndName(TabUdeskTicket tabUdeskTicket, Integer organizationId) {
        tabUdeskTicket.setOrganizationId(organizationId);
        Map<String, Object> orgParam = new HashMap<>(8);
        orgParam.put("type", "id");
        orgParam.put("content", organizationId);
        logger.info("请求udesk企业详情入参id为{}", organizationId);
        String orgDetailStr = UdeskHttpClientUtil.getUdeskJsonFromParam(ORG_DETAIL, orgParam);
        logger.info("请求udesk企业详情返回为{}", orgDetailStr);
        UdeskCompanyDTO orgDetail = JSON.parseObject(orgDetailStr, UdeskCompanyDTO.class);
        if (orgDetail.getCode() == UdeskConstant.UDESK_SUCCESS_CODE) {
            OrganizationDTO organization = orgDetail.getOrganization();
            if (organization != null) {
                tabUdeskTicket.setOrganizationName(organization.getName());
            }
        } else {
            String exceptionRecord = "企业id为" + organizationId + "请求udesk企业详情异常，返回为" + orgDetailStr;
            logger.info(exceptionRecord);
            udeskBaseApiService.saveUdeskException(exceptionRecord);
        }
    }

    /**
     * 填充工单优先级中文名称
     * @param ticket
     * @param tabUdeskTicket
     * @param tapdBug
     */
    private void setPriority(TicketDTO ticket, TabUdeskTicket tabUdeskTicket, TapdBugBaseDTO tapdBug) {
        String priority = ticket.getPriority();
        if (UdeskConstant.UDESK_PRIORITY_NORMAL.equals(priority)) {
            priority = UdeskConstant.TAPD_PRIORITY_NORMAL;
        }
        tabUdeskTicket.setPriority(priority);
        tapdBug.setPriority(tabUdeskTicket.getPriority());
    }

    /**
     * 填充tapd缺陷的自定义字段值
     * @param workspaceId
     * @param tapdBug
     * @param tabUdeskTicket
     */
    private void setTapdCustomField(String workspaceId, TapdBugDTO tapdBug, TabUdeskTicket tabUdeskTicket) {
        TabUdeskTapdRel params = new TabUdeskTapdRel();
        params.setRelType(9);
        params.setWorkspaceId(workspaceId);
        List<TabUdeskTapdRel> udeskTapdRelList = udeskTapdRelService.list(params);
        for (TabUdeskTapdRel tabUdeskTapdRel : udeskTapdRelList) {
            try {
                Method udeskMethod = TabUdeskTicket.class.getMethod(tabUdeskTapdRel.getUdeskIdKey());
                String value = (String) udeskMethod.invoke(tabUdeskTicket);
                Method tapdMethod = TapdBugDTO.class.getMethod(tabUdeskTapdRel.getTapdId(), String.class);
                tapdMethod.invoke(tapdBug, value);
            } catch (Exception e) {
                String exceptionRecord = "创建tapd缺陷设置tapd自定义字段异常" + e.getMessage();
                logger.info(exceptionRecord, e);
                udeskBaseApiService.saveUdeskException(exceptionRecord);
            }
        }
    }

    /**
     * 填充tapd缺陷的自定义字段值
     * @param workspaceId
     * @param tapdBug
     * @param tabUdeskTicket
     */
    private void setTapdCustomField(String workspaceId, TapdBugUpdateDTO tapdBug, TabUdeskTicket tabUdeskTicket) {
        TabUdeskTapdRel params = new TabUdeskTapdRel();
        params.setWorkspaceId(workspaceId);
        params.setRelType(9);
        List<TabUdeskTapdRel> udeskTapdRelList = udeskTapdRelService.list(params);
        for (TabUdeskTapdRel tabUdeskTapdRel : udeskTapdRelList) {
            String udeskIdKey = tabUdeskTapdRel.getUdeskIdKey();
            if ("getFieldNum".equals(udeskIdKey) || "getServiceStaff".equals(udeskIdKey)) {
                continue;
            }
            try {
                Method udeskMethod = TabUdeskTicket.class.getMethod(udeskIdKey);
                String value = (String) udeskMethod.invoke(tabUdeskTicket);
                Method tapdMethod = TapdBugUpdateDTO.class.getMethod(tabUdeskTapdRel.getTapdId(), String.class);
                tapdMethod.invoke(tapdBug, value);
            } catch (Exception e) {
                String exceptionRecord = "更新tapd缺陷设置tapd自定义字段异常" + e.getMessage();
                logger.info(exceptionRecord, e);
                udeskBaseApiService.saveUdeskException(exceptionRecord);
            }
        }
    }

    private void setDescription(TicketDTO ticket, TapdBugBaseDTO tapdBug) {
        String content = "<p>" + ticket.getContent() + "</p>";
        List<AttachmentDTO> attachments = ticket.getAttachments();
        if (attachments != null && attachments.size() > 0) {
            StringBuilder builder = new StringBuilder(content).append("<p>附件:<ul id=udesk_attachments'>");
            for (AttachmentDTO attachment : attachments) {
                builder.append("<li>").append("<a href='" + attachment.getUrl() + "' download='")
                        .append(attachment.getFile_name()).append("'>").append(attachment.getFile_name())
                        .append("</a></li>");
            }
            tapdBug.setDescription(builder.append("</ul></p>").toString());
        } else {
            tapdBug.setDescription(content);
        }
    }

    /**
     * 删除消息数据
     * @param tabUdeskMessage
     * @param id
     * @param bugId
     */
    private void updateTabUdeskMessage(TabUdeskMessage tabUdeskMessage, String id, String bugId) {
        tabUdeskMessage.setBugId(bugId);
        tabUdeskMessage.setTicketId(id);
        tabUdeskMessage.setStatus(UdeskConstant.DELETED);
        udeskMessageService.update(tabUdeskMessage);
    }

    /**
     * 同步tapd缺陷状态值
     * @param id
     * @param ticket
     * @param tapdBug
     * @param workspaceId
     * @param bugId
     */
    private void reSetTapdStatus(String id, TicketDTO ticket, TapdBugDTO tapdBug, String workspaceId, String bugId) {
        TapdBugUpdateDTO tapdBugUpdate = new TapdBugUpdateDTO();
        //tapd缺陷主键ID
        tapdBugUpdate.setId(bugId);
        //标题
        tapdBugUpdate.setTitle(ticket.getSubject());
        tapdBugUpdate.setCurrent_user(UdeskConstant.TPAD_REPORTER);
        //项目ID
        tapdBugUpdate.setProject_id(Integer.valueOf(workspaceId));
        tapdBugUpdate.setWorkspace_id(workspaceId);
        tapdBugUpdate.setReporter(UdeskConstant.TPAD_REPORTER);
        tapdBugUpdate.setStatus(tapdBug.getStatus());

        String updateStr = JSON.toJSONString(tapdBugUpdate);
        logger.info("创建工单单独更新TAPDbug状态入参为{}", updateStr);
        String bugUpdateReturnStr = UdeskHttpClientUtil.postTapdJsonFromJson(TAPD_BUGS, updateStr);
        logger.info("创建工单单独更新TAPDbug状态返回为{}", bugUpdateReturnStr);
        TapdRespBugDTO tapdRespBugUpdate = JSON.parseObject(bugUpdateReturnStr, TapdRespBugDTO.class);
        if (tapdRespBugUpdate.getStatus() != UdeskConstant.TAPD_RESP_SUCCESS_STATUS) {
            String exceptionRecord = "工单id为" + id + "单独更新TAPDbug状态失败，返回为" + updateStr;
            logger.info(exceptionRecord);
            udeskBaseApiService.saveTapdException(exceptionRecord);
        }
    }

    private void updateGicTicketStatus(Integer ticketId, String status) {
        //同步到gic工单状态
        logger.info("同步gic工单状态，ID：{}, 状态：{}", ticketId, status);
        //添加逻辑：如果是关闭状态，则不同步到gic侧
        if (!UdeskStatusEnum.CLOSED.getCode().equals(status)) {
            udeskEnterpriseService.updateStatus(ticketId, status);
        }
    }
}
