package com.gic.udesk.web.controller;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import com.alibaba.fastjson.JSON;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.ano.IgnoreLogin;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.udesk.constant.*;
import com.gic.udesk.dto.gic.GicTicketDTO;
import com.gic.udesk.dto.uploadfile.UploadFileDTO;
import com.gic.udesk.qo.TicketListQO;
import com.gic.udesk.service.GicTicketApiService;
import com.gic.udesk.service.UdeskBaseApiService;
import com.gic.udesk.web.utils.UdeskHttpClientUtil;

@RestController
public class GicTicketController {
    private static final Logger LOGGER = LogManager.getLogger(GicTicketController.class);

    @Autowired
    private UdeskBaseApiService udeskBaseApiService;
    @Autowired
    private GicTicketApiService gicTicketApiService;

    @RequestMapping("/get-gic-service-select")
    @IgnoreLogin
    public RestResponse getGicServiceSelect() {
        return ResultControllerUtils.commonResult(udeskBaseApiService.getGicServiceSelect());
    }

    /**
     * 获取工单模板的自定义字段
     * @param type 1：gic基础业务 2：应用
     * @return
     */
    @RequestMapping("/get-custom-field-by-type")
    @IgnoreLogin
    public RestResponse getCustomFieldFromUdeskTemplates(Integer type) {
        return ResultControllerUtils.commonResult(udeskBaseApiService.getCustomFieldFromUdeskTemplates(type));
    }

    /**
     * 工单回复信息
     * @param ticketId
     * @param currentPage
     * @param pageSize
     * @return
     */
    @RequestMapping("/get-ticket-replies")
    @IgnoreLogin
    public RestResponse getTicketReplies(Integer ticketId, Integer currentPage, Integer pageSize) {
        return ResultControllerUtils.commonResult(gicTicketApiService.getTicketReplies(ticketId, currentPage, pageSize));
    }

    @RequestMapping("/get-ticket-detail")
    @IgnoreLogin
    public RestResponse getTicketDetail(Integer ticketId) {
        return ResultControllerUtils.commonResult(gicTicketApiService.getTicketDetailById(ticketId));
    }

    @RequestMapping("/add-ticket")
    public RestResponse addTicket(GicTicketDTO dto) {
        dto.setTicket_field(JSON.parseObject(dto.getTicketFieldJsonStr(), Map.class));
        dto.setGicEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        dto.setGicUserId(UserDetailUtils.getUserDetail().getUserId());
        dto.setGicUserName(UserDetailUtils.getUserDetail().getUserInfo().getUserName());
        dto.setGicEnterpriseName(UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseName());
        LogUtils.createLog("提交工单", "工单管理");
        return ResultControllerUtils.commonResult(gicTicketApiService.saveTicket(dto));
    }

    @RequestMapping("/list-ticket")
    public RestResponse page(TicketListQO params) {
        params.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(gicTicketApiService.page(params));
    }

    @RequestMapping("/close-ticket")
    @IgnoreLogin
    public RestResponse closeTicket(Integer ticketId) {
        LogUtils.createLog("关闭工单", "工单管理");
        return ResultControllerUtils.commonResult(gicTicketApiService.closeTicket(ticketId));
    }

    @RequestMapping("/ticket-upload-file")
    @IgnoreLogin
    public RestResponse getCustomFieldFromUdeskTemplates(HttpServletRequest request) throws Exception {
        String ticketId = request.getParameter("ticketId");
        if (!StringUtils.isNumeric(ticketId)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "工单ID错误，数值非法");
        }
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        //上传一个文件处理方式，如果是多个，用getFiles
        MultipartFile file = multipartRequest.getFile("file");
        if (file.isEmpty()) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "文件为空");
        }
        LOGGER.info("udesk工单上传附件");
        String replyRespStr = UdeskHttpClientUtil.postUdeskJsonFromJsonForFile("/tickets/upload_file", Integer.parseInt(ticketId), file);
        LOGGER.info("desk工单上传附件返回为{}", replyRespStr);
        UploadFileDTO result = JSON.parseObject(replyRespStr, UploadFileDTO.class);
        if (result.getCode() == 1000) {
            return RestResponse.success();
        }
        return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), result.getMessage());
    }

    /**
     * udesk自定义字段 客服权限
     * @return
     */
    @RequestMapping("/agent-permission-enum")
    @IgnoreLogin
    public RestResponse agentPermissionEnum() {
        return RestResponse.success(AgentPermissionEnum.list());
    }

    /**
     * udesk自定义字段 计算字段的计算结果类型
     * @return
     */
    @RequestMapping("/calculate-result-type-enum")
    @IgnoreLogin
    public RestResponse calculateResultTypeEnum() {
        return RestResponse.success(CalculateResultTypeEnum.list());
    }

    @RequestMapping("/ticket-status-enum")
    @IgnoreLogin
    public RestResponse ticketStatusEnum() {
        return RestResponse.success(UdeskStatusEnum.list());
    }

    /**
     * udesk自定义字段 客户权限
     * @return
     */
    @RequestMapping("/custom-permission-enum")
    @IgnoreLogin
    public RestResponse customPermissionEnum() {
        return RestResponse.success(CustomerPermissionEnum.list());
    }

    /**
     * udesk自定义字段 字段类型
     * @return
     */
    @RequestMapping("/custom-field-content-type-enum")
    @IgnoreLogin
    public RestResponse customFieldContentTypeEnum() {
        return RestResponse.success(CustomFieldContentTypeEnum.list());
    }

}
